/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.*
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.viewpager.widget.ViewPager
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.google.android.material.tabs.TabLayout
import com.kakao.sdk.auth.DefaultTokenManager
import com.kakao.sdk.auth.TokenManagerProvider
import com.kakao.sdk.partner.KakaoPhase
import com.kakao.sdk.partner.sample.internal.PhaseAdapter
import com.kakao.sdk.partner.sample.internal.TabAdapter
import com.kakao.sdk.sample.common.internal.DebugFragment
import io.reactivex.Observable
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.bottom_sheet_phase.view.*

class MainActivity : AppCompatActivity() {
    lateinit var tabAdapter: TabAdapter
    val debugFragment = DebugFragment.newInstance(PartnerApplication.logObservable)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        viewPager.offscreenPageLimit = 4
        tabAdapter = TabAdapter(this, supportFragmentManager)

        viewPager.adapter = tabAdapter

        Observable.fromIterable(tabAdapter.fragments)
            .map { bottomTab.newTab().setText(it.title).setContentDescription(it.title) }
            .doOnNext { bottomTab.addTab(it) }
            .subscribe()

        bottomTab.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabReselected(tab: TabLayout.Tab?) {
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
            }

            override fun onTabSelected(tab: TabLayout.Tab?) {
                viewPager.currentItem = tab!!.position
            }
        })

        viewPager.addOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrollStateChanged(state: Int) {
            }

            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {
            }

            override fun onPageSelected(position: Int) {
                bottomTab.getTabAt(position)?.select()
            }
        })
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_partner_main, menu)
        val preferences = getSharedPreferences("default", Context.MODE_PRIVATE)
        val currentPhase = KakaoPhase.fromName(
            preferences.getString("phase", KakaoPhase.PRODUCTION.phaseName) as String
        )

        val phaseButton = menu?.findItem(R.id.menu_phase)
        phaseButton?.title = currentPhase.phaseName
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.menu_debug -> {
                if (debugFragment.isAdded) return true
                supportFragmentManager.beginTransaction()
                    .addToBackStack("main")
                    .add(
                        R.id.container,
                        debugFragment
                    ).commit()
                return true
            }
            R.id.menu_phase -> {
                showPhaseDialog()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun showPhaseDialog() {
        val dialog = PhaseBottomSheet()
        dialog.show(supportFragmentManager, "phase")
    }
}

class PhaseBottomSheet : BottomSheetDialogFragment() {
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? = inflater.inflate(R.layout.bottom_sheet_phase, container, false)

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        val adapter = PhaseAdapter()
        view.phaseRecyclerView.layoutManager = LinearLayoutManager(context)
        view.phaseRecyclerView.adapter = adapter

        adapter.phaseObservable.doOnNext { changePhase(it) }.subscribe()
    }

    @SuppressLint("ApplySharedPref")
    private fun changePhase(phase: KakaoPhase) {
        TokenManagerProvider.currentManager.clear()
        (TokenManagerProvider.currentManager as DefaultTokenManager).appCache.commit()
        val preferences = context!!.getSharedPreferences("default", Context.MODE_PRIVATE)
        val currentPhase = preferences.getString("phase", KakaoPhase.PRODUCTION.phaseName)
        if (currentPhase == phase.phaseName) {
            dismiss()
            return
        }
        preferences.edit().putString("phase", phase.phaseName).commit()
        val intent = Intent(context!!, MainActivity::class.java)
            .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK)
        context!!.startActivity(intent)
        Runtime.getRuntime().exit(0)
    }
}
